/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import java.nio.charset.Charset;
import java.security.*;
import javax.annotation.*;
import javax.annotation.concurrent.Immutable;
import javax.xml.bind.annotation.XmlRootElement;

import net.java.truelicense.core.codec.*;
import net.java.truelicense.core.io.*;
import org.apache.commons.codec.binary.Base64;

/**
 * A repository for storing authenticated objects (artifacts)
 * with the help of a {@link Codec}.
 *
 * @author Christian Schlichtherle
 */
@Immutable
@XmlRootElement(name = "repository")
public final class BasicRepository
extends RepositoryModel implements Repository {

    private static final Base64 base64 = new Base64();

    @Override public final RepositoryModel model() { return this; }

    private @Nullable byte[] data(
            final @CheckForNull String body,
            final @CheckForNull Charset charset) {
        if (null == body) return null;
        return null == charset ? decode(body) : body.getBytes(charset);
    }

    private static @Nullable byte[] decode(final @CheckForNull String body) {
        return base64.decode(body);
    }

    private @Nullable String body(
            final @CheckForNull byte[] artifact,
            final @CheckForNull Charset charset) {
        if (null == artifact) return null;
        return null == charset ? encode(artifact) : new String(artifact, charset);
    }

    private static @Nullable String encode(final @CheckForNull byte[] data) {
        return base64.encodeToString(data);
    }

    /**
     * Encodes and signs the given {@code artifact}
     * and returns an artifactory for it.
     *
     * @param  codec the codec for encoding the artifact.
     * @param  engine the signature engine.
     * @param  key the private key.
     * @param  artifact the nullable artifact.
     */
    @Override public final <A> Artifactory<A> sign(
            final Codec codec,
            final Signature engine,
            final PrivateKey key,
            final @Nullable A artifact)
            throws Exception {
        final MemoryStore store = new MemoryStore();
        codec.encode(store, artifact);
        final byte[] artifactData = store.data();
        engine.initSign(key);
        engine.update(artifactData);
        final byte[] signatureData = engine.sign();

        final String artifactBody = body(artifactData, Codecs.charset(codec));
        final String signatureBody = encode(signatureData);
        final String algorithm = engine.getAlgorithm();
        final Artifactory<A> artifactory = new EncodedArtifact<A>(codec, store);

        final RepositoryModel model = model();
        model.setArtifact(artifactBody);
        model.setSignature(signatureBody);
        model.setAlgorithm(algorithm);

        return artifactory;
    }

    /**
     * Verifies the signature of the encoded artifact
     * and returns an artifactory for it.
     *
     * @param  codec the codec for decoding the artifact.
     * @param  engine the signature engine.
     * @param  key the public key.
     * @throws IllegalStateException if the state of the model doesn't
     *         match the configuration of the codec or the signature engine.
     * @throws RepositoryIntegrityException if the integrity of this repositoryProvider
     *         with its encoded artifact has been compromised.
     */
    @Override public final Artifactory<?> verify(
            final Codec codec,
            final Signature engine,
            final PublicKey key)
            throws Exception {
        final RepositoryModel model = model();
        if (!engine.getAlgorithm().equalsIgnoreCase(model.getAlgorithm()))
            throw new IllegalArgumentException();
        engine.initVerify(key);
        final Charset charset = Codecs.charset(codec);
        final byte[] artifactData = data(model.getArtifact(), charset);
        engine.update(artifactData);
        if (!engine.verify(decode(model.getSignature())))
            throw new RepositoryIntegrityException();
        return new EncodedArtifact<Object>(codec, artifactData);
    }
}
